@@ -13,12 +13,14 @@ trainControl <- function(method = "boot",
                          summaryFunction = defaultSummary,
                          selectionFunction = "best",
                          preProcOptions = list(thresh = 0.95, ICAcomp = 3, k = 5),
+                         sampling = NULL,
                          index = NULL,
                          indexOut = NULL,
                          timingSamps = 0,
                          predictionBounds = rep(FALSE, 2),
                          seeds = NA,
                          adaptive = list(min = 5, alpha = 0.05, method = "gls", complete = TRUE),
+                         trim = FALSE,
                          allowParallel = TRUE)
 {
   if(is.null(selectionFunction)) stop("null selectionFunction values not allowed")
@@ -50,12 +52,14 @@ trainControl <- function(method = "boot",
        summaryFunction = summaryFunction,
        selectionFunction = selectionFunction,
        preProcOptions = preProcOptions,
+       sampling = sampling,
        index = index,
        indexOut = indexOut,
        timingSamps = timingSamps,
        predictionBounds = predictionBounds,
        seeds = seeds,
        adaptive = adaptive,
+       trim = trim,
        allowParallel = allowParallel)
 }
 
